/**
 *  '$RCSfile: NodeRecord.java,v $'
 *    Purpose: A Class that represents all node info from the database
 *  Copyright: 2000 Regents of the University of California and the
 *             National Center for Ecological Analysis and Synthesis
 *    Authors: Matt Jones
 *    Release: @release@
 *
 *   '$Author: tao $'
 *     '$Date: 2003/04/13 03:09:12 $'
 * '$Revision: 1.6 $'
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package edu.ucsb.nceas.metacat;

/**
 * A utility class that encapsulates a node and its data
 */
public class NodeRecord {
  public long nodeid = -1;
  public long parentnodeid = -1;
  public long nodeindex = -1;
  public String nodename = null;
  public String nodeprefix = null;
  public String nodetype = null;
  public String nodedata = null;

  /**
   * Constructor
   */
  public NodeRecord(long nodeid, long parentnodeid, long nodeindex,
                    String nodetype, String nodename, String nodeprefix, 
                    String nodedata) {
    this.nodeid = nodeid;
    this.parentnodeid = parentnodeid;
    this.nodeindex = nodeindex;
    this.nodename = nodename;
    this.nodeprefix = nodeprefix;
    this.nodetype = nodetype;
    this.nodedata = nodedata;
  }
  
  /** Get functions*/
  public long getNodeId()
  {
    return nodeid;
  }
  
  public long getParentNodeId()
  {
    return parentnodeid;
  }
  
  public long getNodeIndex()
  {
    return nodeindex;
  }
  
  public String getNodeName()
  {
    return nodename;
  }
  
  public String getNodeType()
  {
    return nodetype;
  }
  
  public String getNodePrefix()
  {
    return nodeprefix;
  }
  
  public String getNodeData()
  {
    return nodedata;
  }
  
  /** Method compare two records */
  public boolean contentEquals(NodeRecord record)
  {
    boolean flag = true;
    MetaCatUtil.debugMessage("First nodetype: "+this.nodetype, 40);
    MetaCatUtil.debugMessage("Second nodetype: "+record.getNodeType(), 40);
    MetaCatUtil.debugMessage("First nodename: "+this.nodename, 40);
    MetaCatUtil.debugMessage("Second nodename: "+record.getNodeName(), 40);
    MetaCatUtil.debugMessage("First nodeprefix: "+this.nodeprefix, 40);
    MetaCatUtil.debugMessage("Second nodeprefix: "+record.getNodePrefix(), 40);
    MetaCatUtil.debugMessage("First nodedata: "+this.nodedata, 40);
    MetaCatUtil.debugMessage("Second nodedata: "+record.getNodeData(), 40);
    if ((this.nodename == null && record.getNodeName() != null) ||
        (this.nodename != null && record.getNodeName() == null) ||
        (this.nodename != null && record.getNodeName() != null &&
        !(this.nodename).equals(record.getNodeName())))
    {
      //compare nodename
      flag = false;
    }
    else if ((this.nodetype == null && record.getNodeType() != null) ||
             (this.nodetype != null && record.getNodeType() == null) ||
             (this.nodetype != null && record.getNodeType() != null &&
             !(this.nodetype).equals(record.getNodeType())))
    {
      // compare node type
      flag = false;
    }
    else if ((this.nodeprefix == null && record.getNodePrefix() != null) ||
             (this.nodeprefix != null && record.getNodePrefix() == null) ||
             (this.nodeprefix != null && record.getNodePrefix() != null &&
             !(this.nodeprefix).equals(record.getNodePrefix())))
    {
      // compare node prefix
      flag = false;
    }
    else if ((this.nodedata == null && record.getNodeData() != null) ||
             (this.nodedata != null && record.getNodeData() == null) ||
             (this.nodedata != null && record.getNodeData() != null &&
             !(this.nodedata).equals(record.getNodeData())))
    {
      // compare node data
      flag = false;
    }
    return flag;
    
  }//contentEquals
}
